from asyncio.log import logger
from typing import List
import sys
from netmiko import ConnectHandler, BaseConnection
from pandas import read_csv, DataFrame
from config import host_file, command_file, output_file, log_fomartter
import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
fomartter = logging.Formatter(log_fomartter)


stream = logging.StreamHandler()
stream.setFormatter(fomartter)
logger.addHandler(stream)

file = logging.FileHandler(output_file)
file.setFormatter(fomartter)
logger.addHandler(file)


def read_devices(csv_file: str) -> DataFrame:
    df = read_csv(csv_file)
    return df


def read_commands(command_file: str) -> List[str]:
    with open(command_file) as f:
        return f.readlines()


def run_commands(connection: BaseConnection, command_list: List[str]):    
    for command in command_list:
        if(command.strip()!=""):
            if(command.strip()=="#enable_mode"):
                logger.info("Entering enable mode")
                connection.enable()
            elif(command.strip()=="#end_enable_mode"):
                logger.info("Exiting enable mode")
                connection.exit_enable_mode()
            elif(command.strip()=="#config_mode"):
                logger.info("Entering config mode")               
                connection.config_mode()
            elif(command.strip()=="#end_config_mode"):
                logger.info("Exiting config mode")
                connection.exit_config_mode()
            else:
                output= connection.send_command(command)
                logger.info(output)


df = read_devices(host_file)   
command_list = read_commands(command_file)
with open(output_file, 'w') as file_output:  
    file_output.truncate()
    sys.stdout = file_output
    for index, row in df.iterrows():
        logger.info("Connect to host: " + str(row['host']))
        try:
            with ConnectHandler(**row) as device_connect:
                logger.info("Connect to host : " + str(row['host']) + " success !")
                run_commands(device_connect, command_list)                
        except Exception as ex:
            logger.info(ex)
    sys.stdout.close()
